"""empty message

Revision ID: 895c6b11c356
Revises: 3cbf4759a67e
Create Date: 2024-05-15 16:58:29.359351

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '895c6b11c356'
down_revision = '3cbf4759a67e'
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('user_infos', sa.Column('first_downloaded_at', sa.TIMESTAMP(timezone=True), nullable=True), schema='fansly')
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('user_infos', 'first_downloaded_at', schema='fansly')
    # ### end Alembic commands ###



"""empty message

Revision ID: 160553c5979e
Revises: 895c6b11c356
Create Date: 2024-07-03 11:48:34.442237

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '160553c5979e'
down_revision = '895c6b11c356'
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index('ix_content_media_mass_message_id', 'content_media_asso', ['mass_message_id'], unique=False, schema='fansly')
    op.create_index('ix_content_media_media_id', 'content_media_asso', ['media_id'], unique=False, schema='fansly')
    op.create_index('ix_content_media_message_id', 'content_media_asso', ['message_id'], unique=False, schema='fansly')
    op.create_index('ix_content_media_post_id', 'content_media_asso', ['post_id'], unique=False, schema='fansly')
    op.create_index('ix_content_media_story_id', 'content_media_asso', ['story_id'], unique=False, schema='fansly')
    op.create_index('ix_content_media_mass_message_id', 'content_media_asso', ['mass_message_id'], unique=False, schema='onlyfans')
    op.create_index('ix_content_media_media_id', 'content_media_asso', ['media_id'], unique=False, schema='onlyfans')
    op.create_index('ix_content_media_message_id', 'content_media_asso', ['message_id'], unique=False, schema='onlyfans')
    op.create_index('ix_content_media_post_id', 'content_media_asso', ['post_id'], unique=False, schema='onlyfans')
    op.create_index('ix_content_media_story_id', 'content_media_asso', ['story_id'], unique=False, schema='onlyfans')
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index('ix_content_media_story_id', table_name='content_media_asso', schema='onlyfans')
    op.drop_index('ix_content_media_post_id', table_name='content_media_asso', schema='onlyfans')
    op.drop_index('ix_content_media_message_id', table_name='content_media_asso', schema='onlyfans')
    op.drop_index('ix_content_media_media_id', table_name='content_media_asso', schema='onlyfans')
    op.drop_index('ix_content_media_mass_message_id', table_name='content_media_asso', schema='onlyfans')
    op.drop_index('ix_content_media_story_id', table_name='content_media_asso', schema='fansly')
    op.drop_index('ix_content_media_post_id', table_name='content_media_asso', schema='fansly')
    op.drop_index('ix_content_media_message_id', table_name='content_media_asso', schema='fansly')
    op.drop_index('ix_content_media_media_id', table_name='content_media_asso', schema='fansly')
    op.drop_index('ix_content_media_mass_message_id', table_name='content_media_asso', schema='fansly')
    # ### end Alembic commands ###



from datetime import datetime

from fastapi import APIRouter, Depends, HTTPException, Request, Response
from pydantic import BaseModel
from sqlalchemy import or_, orm, select, update
from sqlalchemy.orm import contains_eager, lazyload, sessionmaker

from ultima_scraper_db.databases.ultima_archive.api.client import UAClient
from ultima_scraper_db.databases.ultima_archive.schemas.templates.site import (
    RemoteURLModel,
)

router = APIRouter(
    prefix="/remote_urls",
    tags=["remote_urls"],
    responses={404: {"description": "Not found"}},
)


class RemoteURLData(BaseModel):
    user_id: int
    host_id: int
    url: str
    uploaded_at: float


@router.get("/{site_name}/{user_id}")
async def get_remote_url(request: Request, site_name: str, user_id: int):
    database_api = UAClient.database_api

    site_api = database_api.get_site_api(site_name)
    async with site_api as site_api:
        remote_url = await site_api.get_remote_url(user_id)
        return remote_url


@router.patch("/")
async def update_remote_url(
    request: Request,
    site_name: str,
    remote_url: RemoteURLData,
):
    database_api = UAClient.database_api

    site_api = database_api.get_site_api(site_name)
    async with site_api as site_api:
        remote_url_db = await site_api.create_or_update_remote_url(
            remote_url.user_id,
            host_id=remote_url.host_id,
            url=remote_url.url,
            uploaded_at=datetime.fromtimestamp(remote_url.uploaded_at),
        )
        return remote_url_db



from typing import TYPE_CHECKING

import requests
from sqlalchemy import ScalarResult, Select, func, or_, select
from sqlalchemy.ext.asyncio import AsyncSession

from ultima_scraper_db.databases.ultima_archive.schemas.management import (
    ServerModel,
    SiteModel,
)
from ultima_scraper_db.managers.database_manager import Schema

if TYPE_CHECKING:
    from ultima_scraper_collection import datascraper_types


class ManagementAPI:
    def __init__(
        self,
        schema: Schema,
        datascraper: "datascraper_types | None" = None,
    ) -> None:
        self.database = schema.database
        self.schema = schema
        self.datascraper = datascraper

    async def __aenter__(self):
        self._session: AsyncSession = self.schema.sessionmaker()
        return self

    async def __aexit__(self, exc_type: None, exc_value: None, traceback: None):

        await self._session.commit()
        await self._session.aclose()

    def get_session(self):
        assert self._session, "Session has not been set"
        return self._session

    async def get_sites(self):
        stmt = select(SiteModel)
        result = await self.get_session().execute(stmt)
        return result.scalars().all()

    async def get_server(self, server_id: int | None, server_name: str | None):
        stmt = select(ServerModel).where(
            or_(
                ServerModel.id == server_id,
                ServerModel.name == server_name,
            )
        )
        result = await self.get_session().scalar(stmt)
        return result
